@@ -7,6 +7,7 @@
 
 #include <algorithm>  // for find_if, count_if
 #include <array>
+#include <bitset>
 #include <iterator>   // for back_inserter
 #include <map>        // for map
 #include <stdexcept>  // for out_of_range
@@ -492,6 +493,11 @@ struct Chain {
   std::vector<Residue>& children() { return residues; }
   const std::vector<Residue>& children() const { return residues; }
 
+  // Returns false only for alternative conformation (microheterogeneity).
+  bool is_first_in_group(const Residue& res) const {
+    return &res == residues.data() || !(&res - 1)->same_group(res);
+  }
+
   // Returns the previous residue or nullptr.
   // Got complicated by handling of multi-conformations / microheterogeneity.
   const Residue* previous_residue(const Residue& res) const {
@@ -569,9 +575,11 @@ struct AtomAddress {
   char altloc = '\0';
 
   AtomAddress() = default;
+  AtomAddress(const std::string& ch, const SeqId& seqid, const std::string& res,
+              const std::string& atom, char alt='\0')
+    : chain_name(ch), res_id({seqid, "", res}), atom_name(atom), altloc(alt) {}
   AtomAddress(const Chain& ch, const Residue& res, const Atom& at)
-    : chain_name(ch.name), res_id(res), atom_name(at.name), altloc(at.altloc)
-  {}
+    : chain_name(ch.name), res_id(res), atom_name(at.name), altloc(at.altloc) {}
 
   bool operator==(const AtomAddress& o) const {
     return chain_name == o.chain_name && res_id.matches(o.res_id) &&
@@ -617,9 +625,10 @@ inline bool atom_matches(const const_CRA& cra, const AtomAddress& addr) {
 struct Connection {
   enum Type { Covale, Disulf, Hydrog, MetalC, None };
   std::string name;
+  std::string link_id;  // _struct_conn.ccp4_link_id (== _chem_link.id)
   Type type = None;
   Asu asu = Asu::Any;
-  AtomAddress atom[2];
+  AtomAddress partner1, partner2;
   double reported_distance = 0.0;
 };
 
@@ -728,7 +737,6 @@ struct Model {
   static const char* what() { return "Model"; }
   std::string name;  // actually an integer number
   std::vector<Chain> chains;
-  std::vector<Connection> connections;
   explicit Model(std::string mname) noexcept : name(mname) {}
 
   // Returns the first chain with given name, or nullptr.
@@ -807,10 +815,6 @@ struct Model {
     return names;
   }
 
-  Connection* find_connection_by_name(const std::string& conn_name) {
-    return impl::find_or_null(connections, conn_name);
-  }
-
   CRA find_cra(const AtomAddress& address) {
     for (Chain& chain : chains)
       if (chain.name == address.chain_name)
@@ -838,6 +842,15 @@ struct Model {
               r && a ? static_cast<int>(a - r->atoms.data()) : -1 }};
   }
 
+  std::bitset<(size_t)El::END> present_elements() const {
+    std::bitset<(size_t)El::END> table;
+    for (const Chain& chain : chains)
+      for (const Residue& res : chain.residues)
+        for (const Atom& a : res.atoms)
+          table.set((int)a.element.elem);
+    return table;
+  }
+
   std::vector<Chain>& children() { return chains; }
   const std::vector<Chain>& children() const { return chains; }
 };
@@ -894,6 +907,7 @@ struct Structure {
   std::vector<Model> models;
   std::vector<NcsOp> ncs;
   std::vector<Entity> entities;
+  std::vector<Connection> connections;
   std::vector<Helix> helices;
   std::vector<Sheet> sheets;
   std::vector<Assembly> assemblies;
@@ -912,6 +926,10 @@ struct Structure {
 
   CoorFormat input_format = CoorFormat::Unknown;
 
+  const SpaceGroup* find_spacegroup() const {
+    return find_spacegroup_by_name(spacegroup_hm, cell.alpha, cell.gamma);
+  }
+
   const std::string& get_info(const std::string& tag) const {
     static const std::string empty;
     auto it = info.find(tag);
@@ -947,6 +965,27 @@ struct Structure {
     return const_cast<Entity*>(gemmi::get_entity_of(sub, entities));
   }
 
+  Connection* find_connection_by_name(const std::string& conn_name) {
+    return impl::find_or_null(connections, conn_name);
+  }
+
+  Connection* find_connection_by_cra(const const_CRA& cra1,
+                                     const const_CRA& cra2) {
+    for (Connection& c : connections)
+      if ((atom_matches(cra1, c.partner1) && atom_matches(cra2, c.partner2)) ||
+          (atom_matches(cra1, c.partner2) && atom_matches(cra2, c.partner1)))
+        return &c;
+    return nullptr;
+  }
+
+  Connection* find_connection(const AtomAddress& a1, const AtomAddress& a2) {
+    for (Connection& c : connections)
+      if ((a1 == c.partner1 && a2 == c.partner2) ||
+          (a1 == c.partner2 && a2 == c.partner1))
+        return &c;
+    return nullptr;
+  }
+
   double get_ncs_multiplier() const {
     size_t given = std::count_if(ncs.begin(), ncs.end(),
                                  [](const NcsOp& o) { return o.given; });
@@ -997,7 +1036,8 @@ inline void Chain::append_residues(std::vector<Residue> new_resi, int min_sep) {
 }
 
 inline void Structure::setup_cell_images() {
-  cell.set_cell_images_from_spacegroup(find_spacegroup_by_name(spacegroup_hm));
+  const SpaceGroup* sg = find_spacegroup();
+  cell.set_cell_images_from_spacegroup(sg);
 
   // Strict NCS from MTRIXn.
   size_t n = cell.images.size();
