@@ -113,17 +113,28 @@ struct Op {
     return r;
   }
 
-  std::array<int, 3> apply_to_hkl(const std::array<int, 3>& hkl) const {
-    std::array<int, 3> r;
+  std::array<double, 3> apply_to_xyz(const std::array<double, 3>& xyz) const {
+    std::array<double, 3> out;
+    for (int i = 0; i != 3; ++i)
+      out[i] = (rot[i][0] * xyz[0] + rot[i][1] * xyz[1] + rot[i][2] * xyz[2] +
+                tran[i]) / Op::DEN;
+    return out;
+  }
+
+  // Miller is defined in the same way in namespace gemmi in unitcell.hpp
+  using Miller = std::array<int, 3>;
+
+  Miller apply_to_hkl(const Miller& hkl) const {
+    Miller r;
     for (int i = 0; i != 3; ++i)
       r[i] = (rot[0][i] * hkl[0] + rot[1][i] * hkl[1] + rot[2][i] * hkl[2])
              / Op::DEN;
     return r;
   }
 
-  double phase_shift(int h, int k, int l) const {
+  double phase_shift(const Miller& hkl) const {
     constexpr double mult = -2 * 3.1415926535897932384626433832795 / Op::DEN;
-    return mult * (h * tran[0] + k * tran[1] + l * tran[2]);
+    return mult * (hkl[0] * tran[0] + hkl[1] * tran[1] + hkl[2] * tran[2]);
   }
 
   std::array<std::array<int, 4>, 4> int_seitz() const {
@@ -929,7 +940,7 @@ template<class Dummy>
 struct Tables_
 {
   static const SpaceGroup main[554];
-  static const SpaceGroupAltName alt_names[27];
+  static const SpaceGroupAltName alt_names[28];
   static const char ccp4_hkl_asu[230];
 };
 
@@ -1501,7 +1512,7 @@ const SpaceGroup Tables_<Dummy>::main[554] = {
 };
 
 template<class Dummy>
-const SpaceGroupAltName Tables_<Dummy>::alt_names[27] = {
+const SpaceGroupAltName Tables_<Dummy>::alt_names[28] = {
   // In 1990's ITfC vol.A changed some of the standard names, introducing
   // symbols 'e' and 'g'. sgtbx interprets these new symbols with
   // option ad_hoc_1992. spglib uses only the new symbols.
@@ -1532,6 +1543,8 @@ const SpaceGroupAltName Tables_<Dummy>::alt_names[27] = {
   {"A e a a", '2', 326}, // A b a a
   {"B b e b", '1', 329}, // B b c b
   {"B b e b", '2', 330}, // B b c b
+  // help with  parsing of unusual setting names that are present in the PDB
+  {"P 21 21 2a", 0, 532}, // P 21212(a)
 };
 
 
@@ -1576,7 +1589,10 @@ inline const SpaceGroup& get_spacegroup_reference_setting(int number) {
                               + std::to_string(number));
 }
 
-inline const SpaceGroup* find_spacegroup_by_name(std::string name) noexcept {
+// the angles alpha and gamma are optional. If provided they are only used
+// to distinguish hexagonal and rhombohedral settings (e.g. for "R 3").
+inline const SpaceGroup* find_spacegroup_by_name(std::string name,
+                                  double alpha=0., double gamma=0.) noexcept {
   if (name[0] == 'H')
     name[0] = 'R';
   const char* p = impl::skip_blank(name.c_str());
@@ -1589,28 +1605,53 @@ inline const SpaceGroup* find_spacegroup_by_name(std::string name) noexcept {
   if (first == '\0')
     return nullptr;
   p = impl::skip_blank(p+1);
+  // change letters to lower case, except the letter after :
+  for (size_t i = p - name.c_str(); i < name.size(); ++i) {
+    if (name[i] >= 'A' && name[i] <= 'Z')
+      name[i] |= 0x20;  // to lowercase
+    else if (name[i] == ':')
+      while (++i < name.size())
+        if (name[i] >= 'a' && name[i] <= 'z')
+          name[i] &= ~0x20;  // to uppercase
+  }
   for (const SpaceGroup& sg : spacegroup_tables::main)
-    if (sg.hm[0] == first && sg.hm[2] == *p) {
-      const char* a = impl::skip_blank(p + 1);
-      const char* b = impl::skip_blank(sg.hm + 3);
-      while (*a == *b && *b != '\0') {
-        a = impl::skip_blank(a+1);
-        b = impl::skip_blank(b+1);
+    if (sg.hm[0] == first) {
+      if (sg.hm[2] == *p) {
+        const char* a = impl::skip_blank(p + 1);
+        const char* b = impl::skip_blank(sg.hm + 3);
+        while (*a == *b && *b != '\0') {
+          a = impl::skip_blank(a+1);
+          b = impl::skip_blank(b+1);
+        }
+        if (*b == '\0' &&
+            (*a == '\0' || (*a == ':' && *impl::skip_blank(a+1) == sg.ext))) {
+          // Change hexagonal settings to rhombohedral if the unit cell angles
+          // are more consistent with the latter.
+          // We have possible ambiguity in the hexagonal crystal family.
+          // For instance, "R 3" may mean "R 3:H" (hexagonal setting) or
+          // "R 3:R" (rhombohedral setting). The :H symbols come first
+          // in the table and are used by default. The ratio gamma:alpha
+          // is 120:90 in the hexagonal system and 1:1 in rhombohedral.
+          // We assume that the 'R' entry follows directly the 'H' entry.
+          if (*a == '\0' && sg.ext == 'H' && gamma < 1.125 * alpha)
+            return &sg + 1;
+          return &sg;
+        }
+      } else if (sg.hm[2] == '1' && sg.hm[3] == ' ') {
+        // check monoclinic short names, matching P2 to "P 1 2 1";
+        // as an exception "B 2" == "B 1 1 2" (like in the PDB)
+        const char* b = sg.hm + 4;
+        if (*b != '1' || (first == 'B' && *++b == ' ' && *++b != '1')) {
+          char end = (b == sg.hm + 4 ? ' ' : '\0');
+          const char* a = impl::skip_blank(p);
+          while (*a == *b && *b != end) {
+            ++a;
+            ++b;
+          }
+          if (*impl::skip_blank(a) == '\0' && *b == end)
+            return &sg;
+        }
       }
-      if (*b == '\0' &&
-          (*a == '\0' || (*a == ':' && *impl::skip_blank(a+1) == sg.ext)))
-        return &sg;
-    } else if (sg.hm[0] == first && sg.hm[2] == '1' && sg.hm[3] == ' ' &&
-               sg.hm[4] != '1') {
-      // check monoclinic short names
-      const char* a = impl::skip_blank(p);
-      const char* b = sg.hm + 4;
-      while (*a == *b && *b != ' ') {
-        a = impl::skip_blank(a+1);
-        ++b;
-      }
-      if (*a == '\0' && *b == ' ')
-        return &sg;
     }
   for (const SpaceGroupAltName& sg : spacegroup_tables::alt_names)
     if (sg.hm[0] == first && sg.hm[2] == *p) {
